from typing import Optional, Tuple

import ansys.fluent.core.quantity as q
from ansys.fluent.core.quantity._constants import _QuantityType
from ansys.fluent.core.quantity.units import parse_temperature_units


class Quantity(float):
    """Quantity instantiates physical quantities using their real values and units. All
    the instances of this class are converted to base SI units system to have
    consistency in arithmetic operations.

    Parameters
    ----------
    value : int | float
        Real value of quantity.
    units : str
        Unit string representation of quantity.
    quantity_map : dict
        Quantity map representation of quantity.
    dimensions : list
        Dimensions representation of quantity.

    Methods
    -------
    to()
        Converts to given unit string.

    Returns
    -------
    Quantity instance.
    """

    def __new__(
        cls, value, units=None, quantity_map=None, dimensions=None, _type_hint=None
    ):
        if (
            (units and quantity_map)
            or (units and dimensions)
            or (quantity_map and dimensions)
        ):
            raise QuantityError.EXCESSIVE_PARAMETERS()

        _units_table = q.UnitsTable()
        _value = float(value)

        if units is not None:
            _unit = units

        if quantity_map:
            units = q.QuantityMap(quantity_map).units
            _unit = units

        if dimensions:
            _dimensions = q.Dimensions(dimensions=dimensions)
            _unit = _dimensions.units

        _, si_multiplier, si_offset = _units_table.si_data(units=_unit)
        _si_value = (_value + si_offset) * si_multiplier

        return float.__new__(cls, _si_value)

    def __init__(
        self, value, units=None, quantity_map=None, dimensions=None, _type_hint=None
    ):
        if (
            (units and quantity_map)
            or (units and dimensions)
            or (quantity_map and dimensions)
        ):
            raise QuantityError.EXCESSIVE_PARAMETERS()

        self._units_table = q.UnitsTable()
        self._value = float(value)

        if units is not None:
            self._unit = units
            self._dimensions = q.Dimensions(units=units)

        if quantity_map:
            units = q.QuantityMap(quantity_map).units
            self._unit = units
            self._dimensions = q.Dimensions(units=units)

        if dimensions:
            self._dimensions = q.Dimensions(dimensions=dimensions)
            self._unit = self._dimensions.units

        self._type = self._units_table.get_type(self._unit)
        if (
            self._type == _QuantityType.temperature
            and _type_hint == _QuantityType.temperature_difference
        ):
            self._type = _QuantityType.temperature_difference

        si_units, si_multiplier, si_offset = self._units_table.si_data(units=self._unit)

        self._si_units = si_units

        # Well, this is going to have to be a hack, but we
        # need to fix the wider design to do this properly
        self._fix_temperature_units()

        self._si_value = (self.value + si_offset) * si_multiplier

    def _arithmetic_precheck(self, __value) -> str:
        """Validate dimensions of quantities.

        Parameters
        ----------
        __value : Quantity | int | float
            Value modifying current quantity object.
        Returns
        -------
        str
            SI unit string of new quantity.
        """
        # Cannot perform operations between quantities with incompatible dimensions
        if isinstance(__value, Quantity) and (self.dimensions != __value.dimensions):
            raise QuantityError.INCOMPATIBLE_DIMENSIONS(self.units, __value.units)
        # Cannot perform operations on a non-dimensionless quantity
        if not isinstance(__value, Quantity) and (not self.is_dimensionless):
            raise QuantityError.INCOMPATIBLE_VALUE(__value)

    def _temp_precheck(self) -> Optional[str]:
        """Validate units for temperature differences.

        Returns
        -------
        str | None
            Units of temperature difference.
        """
        if self.type in [
            _QuantityType.temperature,
            _QuantityType.temperature_difference,
        ]:
            return "delta_K"

    @property
    def value(self):
        """Real value."""
        return self._value

    @value.setter
    def value(self, new_value):
        self._value = new_value

    @property
    def units(self):
        """Unit string."""
        return self._unit

    @property
    def si_value(self):
        """SI conversion value."""
        return self._si_value

    @property
    def si_units(self):
        """SI conversion unit string."""
        return self._si_units

    @property
    def dimensions(self):
        """Dimensions."""
        return self._dimensions.dimensions

    @property
    def is_dimensionless(self) -> bool:
        """Check if quantity is dimensionless."""
        return all([dim == 0.0 for dim in self.dimensions])

    @property
    def type(self):
        """Type."""
        return self._type

    def to(self, to_units: str) -> "Quantity":
        """Perform quantity conversions.

        Parameters
        ----------
        to_units : str
            Desired unit to be converted to.

        Returns
        -------
        Quantity
            Quantity object containing desired quantity conversion.
        """

        if not isinstance(to_units, str):
            raise TypeError("`to_units` should be a `str` type.")

        new_type = None

        if self.type == _QuantityType.temperature_difference:
            new_type = _QuantityType.temperature_difference
            to_units = Quantity._fix_these_temperature_units(
                to_units, ignore_exponent=True
            )

        # Retrieve all SI required SI data and perform conversion
        _, si_multiplier, si_offset = self._units_table.si_data(to_units)
        new_value = (self.si_value / si_multiplier) - si_offset

        new_obj = Quantity(value=new_value, units=to_units, _type_hint=new_type)

        # Confirm conversion compatibility
        self._arithmetic_precheck(new_obj)

        return new_obj

    def __str__(self):
        return f'({self.value}, "{self.units}")'

    def __repr__(self):
        return f'Quantity ({self.value}, "{self.units}")'

    def __pow__(self, __value):
        temp_dimensions = [dim * __value for dim in self.dimensions]
        new_si_value = self.si_value**__value
        new_dimensions = q.Dimensions(dimensions=temp_dimensions)
        return Quantity(value=new_si_value, units=new_dimensions.units)

    def __mul__(self, __value):
        if isinstance(__value, Quantity):
            temp_dimensions = [
                dim + __value.dimensions[idx] for idx, dim in enumerate(self.dimensions)
            ]
            new_si_value = self.si_value * __value.si_value
            new_dimensions = q.Dimensions(dimensions=temp_dimensions)
            new_units = new_dimensions.units
            return Quantity(
                value=new_si_value,
                units=new_units,
                _type_hint=self._determine_new_type(__value),
            )

        if isinstance(__value, (float, int)):
            new_units = self._temp_precheck() or self.si_units
            return Quantity(value=self.si_value * __value, units=new_units)

    def __rmul__(self, __value):
        return self.__mul__(__value)

    def __truediv__(self, __value):
        if isinstance(__value, Quantity):
            temp_dimensions = [
                dim - __value.dimensions[idx] for idx, dim in enumerate(self.dimensions)
            ]
            new_si_value = self.si_value / __value.si_value
            new_dimensions = q.Dimensions(dimensions=temp_dimensions)
            new_units = new_dimensions.units
            result = Quantity(value=new_si_value, units=new_units)
            # HACK
            convert_to_temp_difference = (
                _QuantityType.temperature == result.type
                and __value.type
                in (_QuantityType.temperature, _QuantityType.temperature_difference)
            )
            if convert_to_temp_difference:
                result._type = _QuantityType.temperature_difference
            return result

        if isinstance(__value, (float, int)):
            new_units = self.si_units
            return Quantity(value=self.si_value / __value, units=new_units)

    def __rtruediv__(self, __value):
        return Quantity(__value, "") / self

    def __add__(self, __value):
        self._arithmetic_precheck(__value)
        new_units = self._temp_precheck() or self.si_units
        new_value = float(self) + float(__value)
        return Quantity(value=new_value, units=new_units)

    def __radd__(self, __value):
        return self.__add__(__value)

    def __sub__(self, __value):
        self._arithmetic_precheck(__value)
        new_units = self._temp_precheck() or self.si_units
        new_value = float(self) - float(__value)
        return Quantity(value=new_value, units=new_units)

    def __rsub__(self, __value):
        return self.__sub__(__value)

    def __neg__(self):
        return Quantity(-self.value, self.units)

    def __gt__(self, __value):
        self._arithmetic_precheck(__value)
        return float(self) > float(__value)

    def __ge__(self, __value):
        self._arithmetic_precheck(__value)
        return float(self) >= float(__value)

    def __lt__(self, __value):
        self._arithmetic_precheck(__value)
        return float(self) < float(__value)

    def __le__(self, __value):
        self._arithmetic_precheck(__value)
        return float(self) <= float(__value)

    def __eq__(self, __value):
        self._arithmetic_precheck(__value)
        return float(self) == float(__value)

    def __neq__(self, __value):
        self._arithmetic_precheck(__value)
        return float(self) != float(__value)

    @staticmethod
    def _fix_these_temperature_units(
        units: str, ignore_exponent: bool, units_to_search: Tuple[str] = None
    ) -> str:
        new_units = parse_temperature_units(units, ignore_exponent, units_to_search)
        return " ".join(
            ("delta_" + term[0])
            if (term[1] and not term[0].startswith("delta_"))
            else term[0]
            for term in new_units
        )

    def _fix_temperature_units(self):
        # HACK
        ignore_exponent = self.type == _QuantityType.temperature_difference
        self._unit = Quantity._fix_these_temperature_units(self._unit, ignore_exponent)
        self._si_units = Quantity._fix_these_temperature_units(
            self._si_units, ignore_exponent, ("K",)
        )

    def _determine_new_type(self, other=None):
        # HACK the only concern here is to fix the loss of
        # Temperature Difference information. Return
        # Temperature Difference if it's involved else None
        # such that the caller figures it out in the usual way
        if _QuantityType.temperature_difference in (self.type, other.type):
            return _QuantityType.temperature_difference


class QuantityError(ValueError):
    """Custom quantity errors."""

    def __init__(self, err):
        super().__init__(err)

    @classmethod
    def EXCESSIVE_PARAMETERS(cls):
        return cls(
            "Quantity only accepts 1 of the following parameters: (units) or (quantity_map) or (dimensions)."
        )

    @classmethod
    def INCOMPATIBLE_DIMENSIONS(cls, from_unit, to_unit):
        return cls(f"`{from_unit}` and `{to_unit}` have incompatible dimensions.")

    @classmethod
    def INCOMPATIBLE_VALUE(cls, value):
        return cls(f"`{value}` is incompatible with the current quantity object.")

    @classmethod
    def UNKNOWN_UNITS(cls, unit: str):
        return cls(f"`{unit}` is an unknown or unconfigured unit.")



# quantity "types"
class _QuantityType:
    composite = "Composite"
    derived = "Derived"
    no_type = "No Type"
    temperature = "Temperature"
    temperature_difference = "Temperature Difference"



#
# This is an auto-generated file.  DO NOT EDIT!
#

from ansys.fluent.core.solver.flobject import *

from ansys.fluent.core.solver.flobject import _ChildNamedObjectAccessorMixin

from ansys.fluent.core.solver.flobject import _CreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _NonCreatableNamedObjectMixin

from ansys.fluent.core.solver.flobject import _HasAllowedValuesMixin

from .implicit_bodyforce_treatment import implicit_bodyforce_treatment as implicit_bodyforce_treatment_cls
from .velocity_formulation import velocity_formulation as velocity_formulation_cls
from .physical_velocity_formulation import physical_velocity_formulation as physical_velocity_formulation_cls
from .disable_rhie_chow_flux import disable_rhie_chow_flux as disable_rhie_chow_flux_cls
from .presto_pressure_scheme import presto_pressure_scheme as presto_pressure_scheme_cls
from .first_to_second_order_blending import first_to_second_order_blending as first_to_second_order_blending_cls
from .alternate_diffusion_for_porous_region_solids import alternate_diffusion_for_porous_region_solids as alternate_diffusion_for_porous_region_solids_cls
class numerics_pbns(Group):
    """
    Set numeric options.
    """

    fluent_name = "numerics-pbns"

    child_names = \
        ['implicit_bodyforce_treatment', 'velocity_formulation',
         'physical_velocity_formulation', 'disable_rhie_chow_flux',
         'presto_pressure_scheme', 'first_to_second_order_blending',
         'alternate_diffusion_for_porous_region_solids']

    implicit_bodyforce_treatment: implicit_bodyforce_treatment_cls = implicit_bodyforce_treatment_cls
    """
    implicit_bodyforce_treatment child of numerics_pbns.
    """
    velocity_formulation: velocity_formulation_cls = velocity_formulation_cls
    """
    velocity_formulation child of numerics_pbns.
    """
    physical_velocity_formulation: physical_velocity_formulation_cls = physical_velocity_formulation_cls
    """
    physical_velocity_formulation child of numerics_pbns.
    """
    disable_rhie_chow_flux: disable_rhie_chow_flux_cls = disable_rhie_chow_flux_cls
    """
    disable_rhie_chow_flux child of numerics_pbns.
    """
    presto_pressure_scheme: presto_pressure_scheme_cls = presto_pressure_scheme_cls
    """
    presto_pressure_scheme child of numerics_pbns.
    """
    first_to_second_order_blending: first_to_second_order_blending_cls = first_to_second_order_blending_cls
    """
    first_to_second_order_blending child of numerics_pbns.
    """
    alternate_diffusion_for_porous_region_solids: alternate_diffusion_for_porous_region_solids_cls = alternate_diffusion_for_porous_region_solids_cls
    """
    alternate_diffusion_for_porous_region_solids child of numerics_pbns.
    """



import ansys.fluent.core.quantity as q


class Dimensions(object):
    """Initialize a Dimensions object using a dimensions list or a unit string.

    Parameters
    ----------
    units: str
        Unit string of quantity.
    dimensions: list
        List of dimensions.
    unit_sys: str
        Unit system used for creating units.

    Returns
    -------
    Dimensions instance.
    """

    def __init__(
        self, units: str = None, dimensions: list = None, unit_sys: str = None
    ):
        if units and dimensions:
            raise DimensionsError.EXCESSIVE_PARAMETERS()

        self._units_table = q.UnitsTable()
        unit_sys = unit_sys or self._units_table.unit_systems["SI"]

        if units is not None:
            self._unit = units
            self._dimensions = self._units_to_dim(units=units)

        if dimensions:
            if len(dimensions) > self.max_dim_len():
                raise DimensionsError.EXCESSIVE_DIMENSIONS(len(dimensions))

            self._dimensions, self._unit = self._dim_to_units(
                dimensions=dimensions, unit_sys=unit_sys
            )

    def _dim_to_units(self, dimensions: list, unit_sys: list) -> str:
        """Convert a dimensions list into a unit string.

        Parameters
        ----------
        dimensions : list
            List of unit dimensions.

        unit_sys : list
            Unit system of dimensions.

        Returns
        -------
        str
            Unit string representation of dimensions.
        """
        # Ensure dimensions list contains 9 terms
        dimensions = [
            float(dim)
            for dim in dimensions + ((self.max_dim_len() - len(dimensions)) * [0])
        ]
        units = ""

        # Define unit term and associated value from dimension with dimensions list
        for idx, dim in enumerate(dimensions):
            if dim == 1.0:
                units += f" {unit_sys[idx]}"
            elif dim != 0.0:
                dim = int(dim) if dim % 1 == 0 else dim
                units += f" {unit_sys[idx]}^{dim}"

        return dimensions, self._units_table.condense(units=units)

    def _units_to_dim(
        self, units: str, power: float = None, dimensions: list = None
    ) -> list:
        """Convert a unit string into a dimensions list.

        Parameters
        ----------
        units : str
            Unit string of quantity.
        power : float
            Power of unit string.

        Returns
        -------
        list
            Dimensions representation of unit string.
        """

        power = power or 1.0
        dimensions = dimensions or [0.0] * self.max_dim_len()

        # Split unit string into terms and parse data associated with individual terms
        for term in units.split(" "):
            _, unit_term, unit_term_power = self._units_table.filter_unit_term(term)

            unit_term_power *= power

            # retrieve data associated with fundamental unit
            if unit_term in self._units_table.fundamental_units:
                idx = (
                    self._units_table.dimension_order[
                        self._units_table.fundamental_units[unit_term]["type"]
                    ]
                    - 1
                )
                dimensions[idx] += unit_term_power

            # Retrieve derived unit composition unit string and factor.
            if unit_term in self._units_table.derived_units:
                # Recursively parse composition unit string
                dimensions = self._units_to_dim(
                    units=self._units_table.derived_units[unit_term]["composition"],
                    power=unit_term_power,
                    dimensions=dimensions,
                )

        return dimensions

    @property
    def units(self):
        """Unit string representation of dimensions."""
        return self._unit

    @property
    def dimensions(self):
        """Dimensions list."""
        return self._dimensions

    @staticmethod
    def max_dim_len():
        """Maximum number of elements within a dimensions list."""
        return 9


class DimensionsError(ValueError):
    """Custom dimensions errors."""

    def __init__(self, err):
        super().__init__(err)

    @classmethod
    def EXCESSIVE_PARAMETERS(cls):
        return cls(
            "Dimensions only accepts 1 of the following parameters: (units) or (dimensions)."
        )

    @classmethod
    def EXCESSIVE_DIMENSIONS(cls, len):
        return cls(
            f"The `dimensions` argument must contain 9 values or less, currently there are {len}."
        )



import os

import yaml

from ansys.fluent.core.quantity._constants import _QuantityType
from ansys.fluent.core.quantity.quantity import Quantity, QuantityError  # noqa: F401
from ansys.fluent.core.quantity.units import parse_temperature_units


class UnitsTable(object):
    """Initializes a UnitsTable object with all table values and unit string
    manipulation methods.

    Methods
    -------
    filter_unit_term()
        Separate multiplier, base, and power from a unit term.
    si_data()
        Compute the SI unit string, SI multiplier, and SI offset from a unit string of any type.
    si_conversion()
        Perform SI conversion based on quantity type.
    condense()
        Condenses a unit string by collecting like-terms.
    get_type()
        Returns the type associated with a unit string.

    Returns
    -------
    UnitsTable instance.
    """

    def __init__(self):
        file_path = os.path.relpath(__file__)
        file_dir = os.path.dirname(file_path)
        qc_path = os.path.join(file_dir, "cfg.yaml")

        with open(qc_path, "r") as qc_yaml:
            qc_data = yaml.safe_load(qc_yaml)

        self._dimension_order: dict = qc_data["dimension_order"]
        self._multipliers: dict = qc_data["multipliers"]
        self._unit_systems: dict = qc_data["unit_systems"]
        self._api_quantity_map: dict = qc_data["api_quantity_map"]
        self._fundamental_units: dict = qc_data["fundamental_units"]
        self._derived_units: dict = qc_data["derived_units"]

    def _has_multiplier(self, unit_term: str) -> bool:
        """Check if a unit term contains a multiplier.

        Parameters
        ----------
        unit_term : str
            Unit term of a unit string.

        Returns
        -------
        bool
            Boolean of multiplier within unit_term.
        """
        # Check if the unit term is not an existing fundamental or derived unit.
        return unit_term and not (
            (unit_term in self._fundamental_units) or (unit_term in self._derived_units)
        )

    def _si_map(self, unit_term: str) -> str:
        """Maps unit to SI unit equivalent.

        Parameters
        ----------
        unit_term : str
            Unit term of a unit string.

        Returns
        -------
        term : str
            SI unit equivalent.
        """
        # Retrieve type associated with unit term
        unit_term_type = self._fundamental_units[unit_term]["type"]

        # Find SI unit with same type as unit term
        for term, term_info in self._fundamental_units.items():
            if term_info["type"] == unit_term_type and term_info["factor"] == 1.0:
                return term

    @property
    def api_quantity_map(self):
        """Quantity map values from settings API."""
        return self._api_quantity_map

    @property
    def fundamental_units(self):
        """Fundamental units and properties representing Mass, Length, Time, Current,
        Chemical Amount, Light, Solid Angle, Angle, Temperature and Temperature
        Difference."""
        return self._fundamental_units

    @property
    def derived_units(self):
        """Derived units and properties composed of fundamental units."""
        return self._derived_units

    @property
    def multipliers(self):
        """Multiplier prefixes and respective factors."""
        return self._multipliers

    @property
    def unit_systems(self):
        """Predefined unit systems and units."""
        return self._unit_systems

    @property
    def dimension_order(self):
        """Order of dimensions."""
        return self._dimension_order

    def filter_unit_term(self, unit_term: str) -> tuple:
        """Separate multiplier, base, and power from a unit term.

        Parameters
        ----------
        unit_term : str
            Unit term of a unit string.

        Returns
        -------
        tuple
            Tuple containing multiplier, base, and power of the unit term.
        """
        multiplier = ""
        power = 1.0

        # strip power from unit term
        if "^" in unit_term:
            power = float(unit_term[unit_term.index("^") + 1 :])
            unit_term = unit_term[: unit_term.index("^")]

        base = unit_term

        # strip multiplier and base from unit term
        has_multiplier = self._has_multiplier(unit_term)
        if has_multiplier:
            for mult in self._multipliers:
                if unit_term.startswith(mult):
                    if not self._has_multiplier(unit_term[len(mult) :]):
                        multiplier = mult
                        base = unit_term[len(mult) :]
                        break

        # if we thought it had a multiplier, that's just because the string wasn't
        # a known unit on its own. So if we can't actually find its multiplier then
        # this string is an invalid unit string
        if has_multiplier and not multiplier:
            raise QuantityError.UNKNOWN_UNITS(unit_term)
        return multiplier, base, power

    def si_data(
        self,
        units: str,
        power: float = None,
        si_units: str = None,
        si_multiplier: float = None,
    ) -> tuple:
        """Compute the SI unit string, SI multiplier, and SI offset.

        Parameters
        ----------
        units : str
            Unit string representation of quantity.
        power : float
            Power of unit string
        si_unitsing : str
            SI unit string representation of quantity.
        si_multiplier : float
            SI multiplier of unit string.
        si_offset : float
            SI offset of a unit string.

        Returns
        -------
        tuple
            Tuple containing si_unitsing, si_multiplier and si_offset.
        """

        # Initialize default values
        units = units or " "
        power = power or 1.0
        si_units = si_units or ""
        si_multiplier = si_multiplier or 1.0
        si_offset = (
            self._fundamental_units[units]["offset"]
            if units in self._fundamental_units
            else 0.0
        )

        # Split unit string into terms and parse data associated with individual terms
        for term in units.split(" "):
            unit_multiplier, unit_term, unit_term_power = self.filter_unit_term(term)

            unit_term_power *= power

            si_multiplier *= (
                self._multipliers[unit_multiplier] ** unit_term_power
                if unit_multiplier
                else 1.0
            )

            # Retrieve data associated with fundamental unit
            if unit_term in self._fundamental_units:
                if unit_term_power == 1.0:
                    si_units += f" {self._si_map(unit_term)}"
                elif unit_term_power != 0.0:
                    si_units += f" {self._si_map(unit_term)}^{unit_term_power}"

                si_multiplier *= (
                    self._fundamental_units[unit_term]["factor"] ** unit_term_power
                )

            # Retrieve derived unit composition unit string and factor.
            if unit_term in self._derived_units:
                si_multiplier *= (
                    self._derived_units[unit_term]["factor"] ** unit_term_power
                )

                # Recursively parse composition unit string
                si_units, si_multiplier, _ = self.si_data(
                    units=self._derived_units[unit_term]["composition"],
                    power=unit_term_power,
                    si_units=si_units,
                    si_multiplier=si_multiplier,
                )

        return self.condense(si_units), si_multiplier, si_offset

    def condense(self, units: str) -> str:
        """Condenses a unit string by collecting like-terms.

        Parameters
        ----------
        unitsing : str
            Unit string to be simplified.

        Returns
        -------
        str
            Simplified unit string.
        """
        terms_and_powers = {}
        units = units.strip()

        # Split unit string into terms and parse data associated with individual terms
        for term in units.split(" "):
            _, unit_term, unit_term_power = self.filter_unit_term(term)

            if unit_term in terms_and_powers:
                terms_and_powers[unit_term] += unit_term_power
            else:
                terms_and_powers[unit_term] = unit_term_power

        units = ""

        # Concatenate unit string
        for term, power in terms_and_powers.items():
            if not (power):
                continue
            if power == 1.0:
                units += f"{term} "
            else:
                power = int(power) if power % 1 == 0 else power
                units += f"{term}^{power} "

        return units.rstrip()

    def get_type(self, units: str) -> str:
        """Returns the type associated with a unit string.

        Parameters
        ----------
        units : str
            Unit string of quantity.

        Returns
        -------
        str
            Type of quantity.
        """

        if units == "":
            return _QuantityType.no_type

        if units in self.fundamental_units:
            return self.fundamental_units[units]["type"]

        if units in self.derived_units:
            return _QuantityType.derived

        # HACK
        temperature_units_to_search = ("K", "C", "F", "R")
        if any([temp in units for temp in temperature_units_to_search]):
            terms = parse_temperature_units(
                units,
                ignore_exponent=False,
                units_to_search=temperature_units_to_search,
            )
            if any(is_diff for (_, is_diff) in terms):
                return _QuantityType.temperature_difference
            return _QuantityType.temperature

        return _QuantityType.composite



from typing import Optional, Tuple


def parse_temperature_units(
    units: str, ignore_exponent: bool, units_to_search: Optional[Tuple[str]] = None
) -> list:
    if units_to_search is None:
        units_to_search = ("K", "C", "F", "R")
    units_out = []
    for term in units.split(" "):
        term_parts = term.split("^")
        label = term_parts[0]
        exponent = term_parts[0] if len(term_parts) > 1 else "0"
        is_temp_diff = (
            label
            and (exponent != "0" or ignore_exponent)
            and label[-1] in units_to_search
        )
        units_out.append((term, is_temp_diff))
    return units_out
